@@ -1,7 +1,7 @@
 // -*- c++ -*-
 #ifndef _GLIBMM_PROPERTYPROXY_H
 #define _GLIBMM_PROPERTYPROXY_H
-/* $Id: propertyproxy.h,v 1.1.1.1 2003/01/07 16:58:52 murrayc Exp $ */
+/* $Id: propertyproxy.h,v 1.3.2.1 2005/06/08 16:07:56 murrayc Exp $ */
 
 /* propertyproxy.h
  *
@@ -28,18 +28,38 @@
 namespace Glib
 {
 
+/** A PropertyProxy can be used to get and set the value of an object's property.
+ * There are usually also get and set methods on the class itself, which you might find more convenient.
+ * With the PropertyProxy, you may use either get_value() and set_value(), or operator=() and
+ * operator PropertyType(), like so:
+ * @code
+ * int height = cellrenderer.property_height();
+ * cellrenderer.property_editable() = true;
+ * @endcode
+ *
+ * You may also receive notification when a property's value changes, by connecting to signal_changed().
+ */
 template <class T>
 class PropertyProxy : public PropertyProxy_Base
 {
 public:
   typedef T PropertyType;
 
-  PropertyProxy(Object* obj, const char* name)
+  PropertyProxy(ObjectBase* obj, const char* name)
     : PropertyProxy_Base(obj, name) {}
 
+  /** Set the value of this property.
+   * @param data The new value for the property.
+   */
   void set_value(const PropertyType& data);
+
+  /** Get the value of this property.
+   * @result The current value of the property.
+   */
   PropertyType get_value() const;
 
+  /** Set the value of this property back to its default value
+   */
   void reset_value()
     { reset_property_(); }
 
@@ -51,22 +71,25 @@ public:
 };
 
 
+/** See PropertyProxy().
+ * This property can be written, but not read, so there is no get_value() method.
+ */
 template <class T>
 class PropertyProxy_WriteOnly : public PropertyProxy_Base
 {
 public:
   typedef T PropertyType;
 
-  PropertyProxy_WriteOnly(Object* obj, const char* name)
+  PropertyProxy_WriteOnly(ObjectBase* obj, const char* name)
     : PropertyProxy_Base(obj, name) {}
 
-  void set_value(const PropertyType& data)
-    {
-      PropertyProxy_Base& base = *this;
-      // The downcast to PropertyProxy<T> is safe, and avoids code duplication.
-      static_cast<PropertyProxy<T>&>(base).set_value(data);
-    }
+  /** Set the value of this property.
+   * @param data The new value for the property.
+   */
+  void set_value(const PropertyType& data);
 
+  /** Set the value of this property back to its default value
+   */
   void reset_value()
     { reset_property_(); }
 
@@ -74,22 +97,23 @@ public:
     { this->set_value(data); return *this; }
 };
 
-
+/** See PropertyProxy().
+ * This property can be read, but not written, so there is no set_value() method.
+ */
 template <class T>
 class PropertyProxy_ReadOnly : public PropertyProxy_Base
 {
 public:
   typedef T PropertyType;
 
-  PropertyProxy_ReadOnly(Object* obj, const char* name)
-    : PropertyProxy_Base(obj, name) {}
+  //obj is const, because this should be returned by const accessors.
+  PropertyProxy_ReadOnly(const ObjectBase* obj, const char* name)
+    : PropertyProxy_Base(const_cast<ObjectBase*>(obj), name) {}
 
-  PropertyType get_value() const
-    {
-      const PropertyProxy_Base& base = *this;
-      // The downcast to PropertyProxy<T> is safe, and avoids code duplication.
-      return static_cast<const PropertyProxy<T>&>(base).get_value();
-    }
+  /** Get the value of this property.
+   * @result The current value of the property.
+   */
+  PropertyType get_value() const;
 
   operator PropertyType() const
     { return this->get_value(); }
@@ -120,6 +144,32 @@ T PropertyProxy<T>::get_value() const
   return value.get();
 }
 
+//We previously just static_cast<> PropertyProxy_WriteOnly<> to PropertyProxy<> to call its set_value(), 
+//to avoid code duplication.
+//But the AIX compiler does not like that hack.
+template <class T>
+void PropertyProxy_WriteOnly<T>::set_value(const T& data)
+{
+  Glib::Value<T> value;
+  value.init(Glib::Value<T>::value_type());
+
+  value.set(data);
+  set_property_(value);
+}
+
+//We previously just static_cast<> PropertyProxy_WriteOnly<> to PropertyProxy<> to call its set_value(), 
+//to avoid code duplication.
+//But the AIX compiler does not like that hack.
+template <class T>
+T PropertyProxy_ReadOnly<T>::get_value() const
+{
+  Glib::Value<T> value;
+  value.init(Glib::Value<T>::value_type());
+
+  get_property_(value);
+  return value.get();
+}
+
 #endif /* DOXYGEN_SHOULD_SKIP_THIS */
 
 } // namespace Glib
